@@ -30,18 +30,16 @@ my $i = 0;
 for (@machine){
   my $m = GRID::Machine->new(host => $_, debug => $debug{$_}, );
 
-  if ($m->_x("pi/pi")->result) {
-    $m->chdir("pi/");
-  } 
-  else { 
-    $m->copyandmake(
-      dir => 'pi', 
-      makeargs => 'pi', 
-      files => [ qw{pi.c Makefile} ], 
-      cleanfiles => $cleanup,
-      cleandirs => $cleanup, # remove the whole directory at the end
-    )
-  }
+  $m->copyandmake(
+    dir => 'pi', 
+    makeargs => 'pi', 
+    files => [ qw{pi.c Makefile} ], 
+    cleanfiles => $cleanup,
+    cleandirs => $cleanup, # remove the whole directory at the end
+    keepdir => 1,
+  );
+
+  $m->chdir("pi/");
 
   die "Can't execute 'pi'\n" unless $m->_x("pi")->result;
 
